from textwrap import dedent

import pytest

from clypi import OverflowStyle, wrap


@pytest.mark.parametrize(
    "s,overflow_style,expected",
    [
        ("a", "ellipsis", ["a"]),
        ("a" * 20, "ellipsis", ["a" * 20]),
        ("a" * 21, "ellipsis", ["a" * 19 + "…"]),
        ("a", "wrap", ["a"]),
        ("a" * 20, "wrap", ["a" * 20]),
        ("a" * 21, "wrap", ["a" * 20, "a"]),
        ("a" * 45, "wrap", ["a" * 20, "a" * 20, "a" * 5]),
    ],
)
def test_wrapping(s: str, overflow_style: OverflowStyle, expected: str):
    width = 20
    result = wrap(s, width, overflow_style)
    assert result == expected
    assert len(result) <= width


def test_long_sentence_wrap():
    sentence = dedent(
        """
        This is a very long sentence which should ideally split by the words "sentence",
        "should", "by", "sentence", etc.
        """
    ).strip()
    res = wrap(sentence, width=20, overflow_style="wrap")
    assert res == [
        "This is a very long",
        "sentence which",
        "should ideally split",
        "by the words",
        '"sentence",',
        '"should", "by",',
        '"sentence", etc.',
    ]



import os
import typing as t

from clypi import Command, arg

SOME_ENV_VAR = "SOME_ENV_VAR"
SOME_ENV_VAR2 = "SOME_ENV_VAR2"


class Main(Command):
    foo: float | None = arg(None, env=SOME_ENV_VAR)
    bar: list[int] = arg(env=SOME_ENV_VAR2)


def test_env_var_works(monkeypatch: t.Any):
    monkeypatch.setenv(SOME_ENV_VAR, "-0.1")
    monkeypatch.setenv(SOME_ENV_VAR2, "1,2,3")

    # Just to make sure
    assert os.getenv(SOME_ENV_VAR) == "-0.1"
    assert os.getenv(SOME_ENV_VAR2) == "1,2,3"

    cmd = Main.parse([])
    assert cmd.foo == -0.1
    assert cmd.bar == [1, 2, 3]



from contextlib import suppress
from io import StringIO
from textwrap import dedent

from clypi import Command, Positional, arg, get_config
from tests.prompt_test import replace_stdout


def _assert_stdout_matches(stdout: StringIO, expected: str):
    __tracebackhide__ = True
    stdout_str = stdout.getvalue()
    assert stdout_str.strip() == expected.strip()


def _get_help(
    cmd: type[Command], subcmds: list[str] = [], error: bool = False
) -> StringIO:
    with replace_stdout() as stdout:
        with suppress(SystemExit):
            cmd.parse(
                ["--sdasdasjkdasd"]
                if error
                else [
                    *subcmds,
                    "--help",
                ]
            )

        return stdout


class TestCase:
    def setup_method(self):
        conf = get_config()
        conf.disable_colors = True
        conf.fallback_term_width = 50

    def test_basic_example(self):
        class Main(Command):
            pass

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main
                """
            ),
        )

    def test_basic_example_with_error(self):
        class Main(Command):
            pass

        stdout = _get_help(Main, error=True)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main

                ┏━ Error ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ Unknown option '--sdasdasjkdasd'               ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_with_all_args(self):
        class Subcmd(Command):
            pass

        class Main(Command):
            subcommand: Subcmd
            positional: Positional[str]
            flag: bool = False
            option: int = 5

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [POSITIONAL] [OPTIONS] COMMAND
                
                ┏━ Subcommands ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ subcmd                                         ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                
                ┏━ Arguments ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ [POSITIONAL]                                   ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                
                ┏━ Options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ --flag                                         ┃
                ┃ --option <OPTION>                              ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_with_all_args_and_help(self):
        class Subcmd(Command):
            pass

        class Main(Command):
            subcommand: Subcmd
            positional: Positional[str] = arg(help="Some positional arg")
            flag: bool = arg(False, help="Some flag")
            option: int = arg(5, help="Some option")

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [POSITIONAL] [OPTIONS] COMMAND
                
                ┏━ Subcommands ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ subcmd                                         ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                
                ┏━ Arguments ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ [POSITIONAL]  Some positional arg              ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                
                ┏━ Options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ --flag             Some flag                   ┃
                ┃ --option <OPTION>  Some option                 ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_for_subcommand(self):
        class Subcmd(Command):
            pass

        class Main(Command):
            subcommand: Subcmd
            positional: Positional[str] = arg(help="Some positional arg")
            flag: bool = arg(False, help="Some flag")
            option: int = arg(5, help="Some option")

        stdout = _get_help(Main, subcmds=["subcmd"])
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main subcmd
                """
            ),
        )

    def test_basic_example_group_option(self):
        class Main(Command):
            option: int = arg(5)
            option2: int = arg(5, group="foo")

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [OPTIONS]
               
                ┏━ Options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ --option <OPTION>                              ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
               
                ┏━ Foo options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ --option2 <OPTION2>                            ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_flag_extras(self):
        class Main(Command):
            option: bool = arg(False, short="o", negative="no_option")

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [OPTIONS]
               
                ┏━ Options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ -o, --option/--no-option                       ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_hidden_option(self):
        class Main(Command):
            option: int = arg(5, hidden=True)

        stdout = _get_help(Main)
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [OPTIONS]
                """
            ),
        )

    def test_basic_example_with_long_help(self):
        class Main(Command):
            positional: Positional[str] = arg(
                help="Some positional arg with an insanely long description that will definitely not fit!"
            )

        stdout = _get_help(Main)
        print(stdout.getvalue())
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main [POSITIONAL]
                
                ┏━ Arguments ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ [POSITIONAL]  Some positional arg with an      ┃
                ┃               insanely long description that   ┃
                ┃               will definitely not fit!         ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )

    def test_basic_example_with_inherited_fields(self):
        class Subcmd(Command):
            positional: Positional[str] = arg(inherited=True)
            option: int = arg(inherited=True)

        class Main(Command):
            subcommand: Subcmd
            positional: Positional[str] = arg(help="Some positional arg")
            option: int = arg(5, help="Some option")

        stdout = _get_help(Main, subcmds=["subcmd"])
        print(stdout.getvalue())
        _assert_stdout_matches(
            stdout,
            dedent(
                """
                Usage: main subcmd [POSITIONAL] [OPTIONS]
                
                ┏━ Arguments ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ [POSITIONAL]  Some positional arg              ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                
                ┏━ Options ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
                ┃ --option <OPTION>  Some option                 ┃
                ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
                """
            ),
        )
